;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;
;;; tests.lisp -- package definition
;;; 
;;; Initial version 9/02/2004 by Ben Lee
;;; <blee@common-lisp.net>
;;; 
;;; part of
;;;
;;; Elephant: an object-oriented database for Common Lisp
;;;
;;; Copyright (c) 2004 by Andrew Blumberg and Ben Lee
;;; <ablumberg@common-lisp.net> <blee@common-lisp.net>
;;;
;;; This program is released under the following license
;;; ("GPL").  For differenct licensing terms, contact the
;;; copyright holders.
;;;
;;; This program is free software; you can redistribute it
;;; and/or modify it under the terms of the GNU General
;;; Public License as published by the Free Software
;;; Foundation; either version 2 of the License, or (at
;;; your option) any later version.
;;;
;;; This program is distributed in the hope that it will be
;;; useful, but WITHOUT ANY WARRANTY; without even the
;;; implied warranty of MERCHANTABILITY or FITNESS FOR A
;;; PARTICULAR PURPOSE. See the GNU General Public License
;;; for more details.
;;;
;;; The GNU General Public License can be found in the file
;;; LICENSE which should have been distributed with this
;;; code.  It can also be found at
;;;
;;; http://www.opensource.org/licenses/gpl-license.php
;;;
;;; You should have received a copy of the GNU General
;;; Public License along with this program; if not, write
;;; to the Free Software Foundation, Inc., 59 Temple Place,
;;; Suite 330, Boston, MA 02111-1307 USA
;;;

(defpackage elephant-tests
  (:nicknames ele-tests :ele-tests)
  (:use common-lisp elephant rt)
  (:import-from :ele 
		with-buffer-streams
		serialize
		deserialize)
  #+cmu  
  (:import-from :pcl
		finalize-inheritance
		slot-definition-name
		slot-makunbound-using-class
		class-slots)
  #+sbcl 
  (:import-from :sb-mop 
		finalize-inheritance
		slot-definition-name
		slot-makunbound-using-class
		class-slots)
  #+allegro
  (:import-from :clos
		finalize-inheritance
		slot-definition-name
		slot-makunbound-using-class
		class-slots)
  #+openmcl
  (:import-from :ccl
		finalize-inheritance
		slot-definition-name
		slot-makunbound-using-class
		class-slots)
  #+lispworks  
  (:import-from :clos
		finalize-inheritance
		slot-definition-name
		slot-makunbound-using-class
		class-slots)
  )

(in-package :ele-tests)

;; Putting this in to make the test work; I have no idea what it means...
(deftype array-or-pointer-char () '(or array t))


(defvar *testdb-path* 
  ;;"/usr/local/share/common-lisp/elephant-0.2/tests/testdb"
  (namestring
   (merge-pathnames 
    #p"tests/testdb/" 
    (asdf:component-pathname (asdf:find-system 'elephant-tests)))))

(defvar *sleepycatdb-path* 
  ;;"/usr/local/share/common-lisp/elephant-0.2/tests/testdb"
  (namestring
   (merge-pathnames 
    #p"tests/testsleepycat/" 
    (asdf:component-pathname (asdf:find-system 'elephant-tests)))))

(defvar *testpg-path*
'(:postgresql "localhost.localdomain" "test" "postgres" ""))

(defvar *testsqlite3-path*
;; This is of the form '(filename &optional init-function),
;; and using :memory: as a file name will get you an completely in-memory system...
;; '(":memory:")
 '(:sqlite3 "sqlite3-test.db")
)

(defvar *test-path-primary*
  *testpg-path*
)
(defvar *test-path-secondary*
  *testdb-path*  
)


(defun do-all-tests()
  (progn
    (do-all-tests-spec *testdb-path*)
    (do-all-tests-spec *testpg-path*)
    ))

(defun do-crazy-pg-tests()
  (open-store *testpg-path*)
  (do-test 'indexed-btree-make)
  (do-test 'add-indices)
  (do-test 'test-indices)
  (do-test 'indexed-put)
  (do-test 'indexed-get)
  (close-store)
  )

(defun do-migrate-test-spec(spud)
  (with-open-store(spud)
    (let ((*auto-commit* nil))
      (do-test 'migrate1))))

(defun do-all-tests-spec(spec)
  (with-open-store (spec)
    (let ((*auto-commit* nil))
      (do-tests))))

(defun find-slot-def (class-name slot-name)
  (find-if #'(lambda (slot-def)
	       (eq (slot-definition-name slot-def) slot-name))
	   (class-slots (find-class class-name))))

(defmacro finishes (&body body)
  `(handler-case
    (progn ,@body)
    (error () nil)
    (warning () t)
    (condition () t)
    (:no-error (&rest rest) (declare (ignore rest)) t)))

(defmacro signals-condition (&body body)
  `(handler-case
    (progn ,@body)
    (condition () t)
    (:no-error (&rest rest) (declare (ignore rest)) nil)))

(defmacro signals-error (&body body)
  `(handler-case
    (progn ,@body)
    (error () t)
    (:no-error (&rest rest) (declare (ignore rest)) nil)))

(defmacro is-not-null (&body body)
  `(not (null (progn ,@body))))

(defmacro are-not-null (&rest forms)
  `(values
    ,@(loop for form in forms
	    collect `(is-not-null ,form))))
